pub enum FileError {
FailedToExtractString,
InsertLocationNotFound,
}Expand description
An error indicating something went wrong with a Rivia File operation
Variants
FailedToExtractString
An error indicating that a regex string extraction failed.
InsertLocationNotFound
An error indicating that the insert location was not found