pub enum StringError {
FailedToString,
}Expand description
An error indicating something went wrong with a Rivia String operation
Variants
FailedToString
An error indicating a failure to convert the file value to a string.